
using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Forms;
using GenerarFicheroLog;

namespace TarjetaAdquisicionPic
{
    public class TarjetaPic
    {
        #region Variables

        /// <summary> Constantes generales
        /// </summary>
        public const int ID_PC = 1;
        public const int KEEP_ALIVE_OFF = 0;
        private const int PUERTOREMOTO = 16000;
        public const int LENG = 12;
        private const int SERIE = 0, RED = 1;

        /// <summary> Variables Generales
        /// </summary>
        private int medioTrasmision = -1;
        private int velocidadBps;
        private string puertoCom;
        private byte idPic, idPc;
        private bool ipFija, waitRecepcion = false;
        private byte[] ipPic = new byte[8];
        private byte[] tramaReceive = new byte[LENG + 2];
        private SerialPort sp;
        private Thread ThreadCliente;
        private UdpClient cliente;

        /// <summary> Constantes necesarias para manejo de tramas
        /// </summary>
        public const int ECHO = 0;
        public const int ACK_ECHO = 1;
        public const int READ_IN = 2;
        public const int SEND_IN = 3;
        public const int WRITE_IN = 4;
        public const int ACK_WRITE = 5;
        public const int NO_ACK_WRITE = 6;
        public const int STAND_BYE = 7;
        public const int POLLING = 8;
        public const int EVENT = 9;
        public const int STOP = 10;
        public const int PortSerie = 20;
        public const int PortEthernet = 30;
        public const int WRITE_IN_ONE = 11;
        public const int ACK_WRITE_ONE = 12;
        public const int NO_ACK_WRITE_ONE = 13;


        #endregion

        #region Eventos


        public delegate void manejadorEventoCambioEntradas(byte[] datos, int secuencia);
        public event manejadorEventoCambioEntradas EventoCambioEntradas;

        public delegate void manejadorEventoRespuestaEco(bool flag_eco);
        public event manejadorEventoRespuestaEco EventoRespuestaEco;

        public delegate void manejadorEventoRespuestaWrite(bool flag_write);
        public event manejadorEventoRespuestaWrite EventoRespuestaWrite;

        public delegate void manejadorEventoRespuestaNoWrite(bool flag_NoWrite);
        public event manejadorEventoRespuestaNoWrite EventoRespuestaNoWrite;

        public delegate void manejadorEventoRespuestaWriteOne(bool flag_WriteOne);
        public event manejadorEventoRespuestaWriteOne EventoRespuestaWriteOne;

        public delegate void manejadorEventoRespuestaNoWriteOne(bool flag_NoWriteOne);
        public event manejadorEventoRespuestaNoWriteOne EventoRespuestaNoWriteOne;

        #endregion

        #region Constructores

        /// <summary>
        /// Configuración Modo Serie
        /// </summary>
        /// <param name="puertoCom"> Tipo String Ej: COM1</param>
        /// <param name="velocidadBps"> Valores Comprendidos entre 9600 y 256000, y estandares utilizados en comunicaciones serie </param>
        /// <param name="idPic"> ID del PIC, si es 0 usamos ID por Defecto</param>
        /// <param name="idPc"> ID del PC</param>
        public TarjetaPic(string puertoCom, int velocidadBps, byte idPic, byte idPc)
        {
            this.medioTrasmision = SERIE;
            this.velocidadBps = velocidadBps;
            this.puertoCom = puertoCom;
            this.idPic = idPic;
            this.idPc = idPc;
        }


        /// <summary>
        /// Configura Modo Red, con IP por defecto
        /// </summary>
        /// <param name="puertoCom"> Tipo String Ej: COM1</param>
        /// <param name="idPic">ID del PIC, si es 0 usamos ID por Defecto</param>
        /// <param name="idPc"> ID del PC</param>
        public TarjetaPic(string puertoCom, byte idPic, byte idPc)
        {
            this.medioTrasmision = RED;
            this.puertoCom = puertoCom;
            ipFija = false;
            this.idPic = idPic;
            this.idPc = idPc;
        }

        /// <summary>
        /// Configura Modo Red, con IP fija
        /// </summary>
        /// <param name="puertoCom"> Tipo String Ej: COM1 </param>
        /// <param name="ip"> Array de bytes con contiene la IP y la mascara de Red</param>
        /// <param name="idPic">ID del PIC, si es 0 usamos ID por Defecto</param>
        /// <param name="idPc"> ID del PC</param>
        public TarjetaPic(string puertoCom, byte[] ip, byte idPic, byte idPc)
        {
            this.medioTrasmision = RED;
            this.puertoCom = puertoCom;
            ipPic = ip;
            ipFija = true;
            this.idPic = idPic;
            this.idPc = idPc;
        }

        #endregion

        #region GetSet

        public int getMedio() { return medioTrasmision; } // 0=SERIE, 1=RED

        #endregion

        #region Metrodos

        /// <summary>
        /// Crea la trama necesaria para establecer el medio de comunicación que se utilizara
        /// en el PIC para la comunicación con el PC.
        /// Requisito haberse creado con antelación el objeto TarjetaPic.
        /// </summary>
        public void EstablecerMedio()
        {
            if (medioTrasmision == -1)
            {
                MessageBox.Show("No se ha creado la instancia del objeto TarjetaPic");
            }
            else
            {
                byte[] trama = new byte[LENG];
                sp = new SerialPort(puertoCom, 9600);

                try
                {
                    if (!sp.IsOpen)
                    {
                        sp.Open();
                        trama[0] = idPc;
                        trama[1] = idPic;

                        switch (medioTrasmision)
                        {
                            // Formamos la trama para indicar al PIC que inicio la comunicación en Serie
                            case SERIE:
                                trama[2] = PortSerie;
                                if (VelocidadRs232(velocidadBps) == 0)
                                {
                                    throw new System.Exception("Valor de Velocidad Incorrecto");
                                }
                                trama[3] = VelocidadRs232(velocidadBps);

                                break;

                            // Formamos la trama para indicar al PIC que inicio la comunicación en Red
                            case RED:

                                trama[2] = PortEthernet;
                                if (ipFija == true)
                                {
                                    for (int i = 0; i < 8; i++)
                                    {
                                        trama[i + 3] = ipPic[i];
                                    }
                                }
                                else trama[3] = 0;

                                break;
                        }
                        sp.Write(trama, 0, trama.Length);
                        sp.Close();
                    }
                    else MessageBox.Show("El puerto Serie que intenta utilizar se encuentra abierto");
                }
                catch
                {
                    MessageBox.Show("Error al intentar establecer medio");
                }
            }
        }

        /// <summary>
        /// Función que dada la velocidad de comunicación serie retorna el valor un valor reconocido para el PIC
        /// a la hora de establecer la velocidad con la que se comunicara con el PC.
        /// </summary>
        /// <param name="velocidadBps">Tipo Entero comprendido entre 9600 y 256000, en caso error retorna 0</param>
        /// <returns></returns>
        private byte VelocidadRs232(int velocidadBps)
        {
            switch (velocidadBps)
            {
                case 9600: return 1;
                case 14400: return 2;
                case 19200: return 3;
                case 38400: return 4;
                case 57600: return 5;
                case 115200: return 6;
                case 256000: return 7;
            }
            return 0;
        }

        /// <summary>
        /// Funcion que realiza la conexion entre PC y PIC, una vez instanciado el Objeto.
        /// </summary>
        public void Conectar()
        {

            if (medioTrasmision == -1)
            {
                MessageBox.Show("No se ha establecido el Medio", "Información", MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
            else
            {
                switch (medioTrasmision)
                {
                    case SERIE:
                        sp = new SerialPort(puertoCom, velocidadBps);

                        if (!sp.IsOpen)
                        {
                            try
                            {
                                sp.Open();
                                sp.Encoding = System.Text.Encoding.GetEncoding(1252);
                                /* Se ha realizado el cambio de codificación ya que la que usa por depecto son 7 bits.
                                 * Por lo tanto, hay que hacer un cambio de codificacion para que pueda reconocer los 8 bits y 
                                 * asi poder utilizar los 2 bytes para el contador.
                                 * Si no se realiza el cambio de codificación al recibir los datos en el PC se observa el valor ?,
                                 * que en decimal es 63, que es el valor que viene definido en caso de error de paridad.
                                 * */
                            }
                            catch
                            {
                                MessageBox.Show("Error en la apertura del Puerto COM", "Información",
                                 MessageBoxButtons.OK, MessageBoxIcon.Error);
                            }
                        }
                        break;
                    case RED:
                        try
                        {

                            ThreadCliente = new Thread(new ThreadStart(start_Cliente));
                            //ThreadCliente.IsBackground = true;
                            ThreadCliente.Start();

                        }
                        catch
                        {
                            MessageBox.Show("Error al iniciar el cliente", "Información", MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
                        }
                        break;
                }

            }
        }

        /// <summary> Realiza la desconexion con el PIC.
        /// </summary>
        public void Desconectar()
        {
            waitRecepcion = false;
            switch (medioTrasmision)
            {
                case SERIE:
                    try
                    {
                        sp.DiscardInBuffer();
                        sp.Dispose();
                        sp.Close();
                    }
                    catch
                    {
                        MessageBox.Show("Error al cerrar el Puerto COM", "Información",
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    break;
                case RED:
                    try
                    {
                        if (ThreadCliente.IsAlive)
                        {
                            ThreadCliente.Interrupt();
                            cliente.Close();
                        }
                        else if (ThreadCliente.IsAlive == false)
                        {
                            cliente.Close();
                        }
                    }
                    catch
                    {
                        MessageBox.Show("No es posible parar el servicio", "Información", MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
                    }
                    break;
            }
        }

        /// <summary>
        /// Función que estable el modo que se desea trabajar en el PIC, ya sea, POLLING, EVENT o STOP.
        /// </summary>
        /// <param name="modo">Valor entero que establece el modo. STOP=10, EVENT=9, POLLING=8.</param>
        /// <param name="umbral">Valor de umbral de los cambios en los PINES I/O.</param>
        /// <param name="muestreo">Valores Posibles de Muestreo
        ///                         10,4 useg --> 1
        ///                         20,9 useg --> 2
        ///                         41,9 useg --> 3
        ///                         83,8 useg --> 4
        ///</param>
        /// <param name="keepAlive">KeepAlive OFF=0, ON=>1, en modo ON se establecen 
        ///                         los segundos que se quieren recibiar datos
        /// </param>
        public void EstablecerModo(int modo, byte umbral, int muestreo, int keepAlive)
        {
            if (medioTrasmision == -1)
            {
                MessageBox.Show("No se ha creado la instancia del objeto TarjetaPic");
            }
            else if (muestreo < 1 || muestreo > 4)
            {
                MessageBox.Show("Valor de Muestreo fuera de rango, valores 1-4");
            }
            else
            {
                byte[] trama = new byte[LENG];
                trama[0] = idPic;
                trama[1] = idPc;
                trama[2] = 12;

                trama[4] = Convert.ToByte(muestreo);
                trama[5] = umbral;
                trama[6] = Convert.ToByte(keepAlive);
                switch (modo)
                {
                    case STOP:
                        trama[3] = STOP;
                        break;
                    case POLLING:
                        trama[3] = POLLING;
                        break;
                    case EVENT:
                        trama[3] = EVENT;
                        break;

                }
                switch (medioTrasmision)
                {
                    case SERIE:
                        sp.Write(trama, 0, trama.Length);
                        sp.DataReceived += new SerialDataReceivedEventHandler(recepcion_rs232);
                        break;
                    case RED:
                        int count = cliente.Send(trama, trama.Length);

                        if (count == 0)
                        {
                            MessageBox.Show("El mensaje no ha podido ser enviado.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        break;
                }
            }
        }

        /// <summary>
        /// Función que envía una trama ECO al PIC.
        /// </summary>
        public void enviarEcho()
        {
            if (medioTrasmision == -1)
            {
                MessageBox.Show("No se ha creado la instancia del objeto TarjetaPic");
            }
            else if (!waitRecepcion)
            {
                waitRecepcion = true;
                byte[] tramaEcho = { idPic, idPc, 4, ECHO, 0, 0, 0, 0, 0, 0, 0, 0 };
                switch (medioTrasmision)
                {
                    case SERIE:
                        sp.Write(tramaEcho, 0, tramaEcho.Length);
                        sp.DataReceived += new SerialDataReceivedEventHandler(recepcion_rs232);
                        break;
                    case RED:
                        int count = cliente.Send(tramaEcho, tramaEcho.Length);
                        if (count == 0)
                        {
                            MessageBox.Show("El mensaje no ha podido ser enviado.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        break;
                }
            }
        }

        

        /// <summary>
        /// Función que envía una trama READ_IN al PIC
        /// </summary>
        public void enviarRead()
        {
            if (medioTrasmision == -1)
            {
                MessageBox.Show("No se ha creado la instancia del objeto TarjetaPic");
            }
            else if (!waitRecepcion)
            {
                waitRecepcion = true;
                byte[] tramaTest = { 1, ID_PC, 1, READ_IN, 0, 0, 0, 0, 0, 0, 0, 0 };
                switch (medioTrasmision)
                {
                    case SERIE:
                        sp.Write(tramaTest, 0, tramaTest.Length);
                        sp.DataReceived += new SerialDataReceivedEventHandler(recepcion_rs232);
                        waitRecepcion = true;
                        break;
                    case RED:
                        int count = cliente.Send(tramaTest, tramaTest.Length);

                        if (count == 0)
                        {
                            MessageBox.Show("El mensaje no ha podido ser enviado.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        break;
                }
            }
        }

        
        /// <summary>
        /// Función que envía una trama WRITE_IN al PIC.
        /// </summary>
        /// <param name="datos">Array de bytes que debe contener el estado que se desea tener en los PINES. El
        /// tramaño que debe de tener este array tiene que se de 4.</param>
        public void setSalidas(BitArray datosOut) //bitarray
        {
            byte[] datos = new byte[4];

            datosOut.CopyTo(datos, 0);

            if (medioTrasmision == -1)
            {
                MessageBox.Show("No se ha creado la instancia del objeto TarjetaPic");
            }
            //else if (datos.Length != 4)
            //{
            //    MessageBox.Show("El tamaño de los datos no son correctos, tamaño correcto 4");
            //}
            else if (!waitRecepcion)
            {
                waitRecepcion = true;
                byte[] trama = new byte[LENG];
                trama[0] = idPic;
                trama[1] = idPc;
                trama[2] = 12;
                trama[3] = WRITE_IN;
                for (int i = 0; i < 4; i++)
                    trama[4 + i] = datos[i];
                switch (medioTrasmision)
                {
                    case SERIE:
                        sp.Write(trama, 0, trama.Length);
                        break;
                    case RED:
                        int count = cliente.Send(trama, trama.Length);
                        if (count == 0)
                        {
                            MessageBox.Show("El mensaje no ha podido ser enviado.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        break;
                }
            }
        }
        

        /// <summary>
        /// Realiza el cambio de estado de la puerta deseada. Hay disponibles 30 salidas:
        ///     PUERTA A --> 0    -   5     
        ///     PUERTA E --> 8    -   15
        ///     PUERTA H --> 16   -   23
        ///     PUERTA J --> 24   -   31
        ///     
        ///     NOTA:
        ///     
        ///     ¡¡IMPORTANTE!! Los valores 6 y 7 no son validos, ya que no corresponde a 
        ///     a salidas validas.
        /// </summary>
        /// <param name="num_puerta"> Numero de la puerta a cambiar.</param>
        public void setSalida(byte num_puerta, bool estado )
        {
            if (medioTrasmision == -1)
            {
                MessageBox.Show("No se ha creado la instancia del objeto TarjetaPic");
            }
            else if (num_puerta >= 0 && num_puerta < 32)
            {
                if (num_puerta == 6 || num_puerta == 7)
                {
                    MessageBox.Show("El valor de numero de puerta no es correcto, ver ayuda", "Información", MessageBoxButtons.OK,
                                        MessageBoxIcon.Error);
                }
                else if (!waitRecepcion)
                {
                    waitRecepcion = true;
                    //BitArray puerta = new BitArray(32);
                    byte[] trama = new byte[LENG];
                    byte[] pr_tr = new byte[4];
                    trama[0] = idPic;
                    trama[1] = idPc;
                    trama[2] = 12;
                    trama[3] = WRITE_IN_ONE;

                    //for (int i = 0; i < 32; i++) { puerta[i] = false; }
                    //puerta[num_puerta] = true;
                    //puerta.CopyTo(pr_tr, 0);

                    //for (int i = 0; i < 4; i++) { trama[i + 4] = pr_tr[i]; }
                    trama[4] = num_puerta;
                    trama[5] = Convert.ToByte(estado);

                    switch (medioTrasmision)
                    {
                        case SERIE:
                            sp.Write(trama, 0, trama.Length);
                            break;
                        case RED:
                            int count = cliente.Send(trama, trama.Length);
                            if (count == 0)
                            {
                                MessageBox.Show("El mensaje no ha podido ser enviado.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            }
                            break;
                    }
                }
            }
        }



        /// <summary>
        /// Función que se ejecuta al iniciar el hilo de sockets, encargada de recibir los datos por UDP.
        /// </summary>
        private void start_Cliente()
        {
            //proceso recepcion en hilo
            byte[] data = new byte[LENG];
            string ip;
            if (ipFija)
            {
                ip = ipPic[0] + "." + ipPic[1] + "." + ipPic[2] + "." + ipPic[3];
            }
            else
            {
                ip = "192.168.2.254";
            }

            cliente = new UdpClient(PUERTOREMOTO);

            try
            {
                cliente.Connect(ip, PUERTOREMOTO);
                IPEndPoint pic = new IPEndPoint(IPAddress.Parse(ip), PUERTOREMOTO);

                while (true)
                {
                    data = cliente.Receive(ref pic);
                    if (data.Length > 0)
                    {
                        //Thread.Sleep(20);
                        analizar_trama(data);
                    }
                }
            }
            catch (ThreadAbortException)
            {
                cliente.Close();
                ThreadCliente.Abort();
            }
            catch
            {
                MessageBox.Show("Proceso Recepcion: Ha ocurrido un error.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }



        /// <summary>Función encargada de recibir los datos por puerto Serie.
        /// </summary>
        /// <param name="s"></param>
        /// <param name="e"></param>
        public void recepcion_rs232(object s, EventArgs e)
        {
            try
            {
                for (int i = 0; i < tramaReceive.Length; i++)
                {
                    tramaReceive[i] = Convert.ToByte(sp.ReadByte());
                }
                analizar_trama(tramaReceive);
            }
            catch { }

        }

        /// <summary>Analiza la trama, identifica el tipo y se encarga de comprobar el evento 
        /// que ha sido provocado.
        /// </summary>
        /// <param name="data"> Datos recibidos por la tarjeta</param>
        private void analizar_trama(byte[] data)
        {

            int id_trama = data[3], secuencia;
            string binario;
            byte[] estados_puertas = new byte[8];
            byte[] temp = new byte[2];

            for (int i = 0; i < 8; i++)
            {
                estados_puertas[i] = data[i + 4];
            }
            waitRecepcion = false;

            /* Almacenamos los valores del contador contenido en la trama en el array temp,
            * para luego convertirlo en binario, para convertirlo en su numeracion correcta */

            temp[0] = data[13];
            temp[1] = data[12];
            binario = DecimaltoBinaryString(temp);
            secuencia = BinarytoDecimal(binario);

            switch (id_trama)
            {
                case ACK_ECHO:
                    if (EventoRespuestaEco != null)
                    {
                        EventoRespuestaEco(true);
                    }
                    break;
                case SEND_IN:
                    if (EventoCambioEntradas != null)
                    {
                        EventoCambioEntradas(estados_puertas, secuencia);
                    }
                    break;
                case ACK_WRITE:
                    if (EventoRespuestaWrite != null)
                    {
                        EventoRespuestaWrite(true);
                    }
                    break;
                case NO_ACK_WRITE:
                    if (EventoRespuestaNoWrite != null)
                    {
                        EventoRespuestaNoWrite(true);
                    }
                    break;
                case ACK_WRITE_ONE:
                    if (EventoRespuestaWriteOne != null)
                    {
                        EventoRespuestaWriteOne(true);
                    }
                    break;
                case NO_ACK_WRITE_ONE:
                    if (EventoRespuestaNoWriteOne != null)
                    {
                        EventoRespuestaNoWriteOne(true);
                    }
                    break;
            }
        }

        /// <summary> Convirte una array de bytes en Codigo Binario 
        /// </summary>
        /// <param name="binario"></param>
        /// <returns></returns>

        #region funcionesCambioABinario

        private int BinarytoDecimal(String binario)
        {
            return Convert.ToInt32(binario, 2);
        }

        public String DecimaltoBinaryString(byte[] values)
        {

            String s = "";

            for (int i = values.Length - 1; i >= 0; i--)
            {
                s += ByteToBitString(values[i]);
            }
            return s;

        }

        private string ByteToBitString(byte value)
        {

            string s = "";

            for (int i = 0; i < 8; i++)
            {
                s += (value & 0x80) > 0 ? "1" : "0";

                value <<= 1;
            }
            return s;

        }
        #endregion



        #endregion

    }

}
